package com.fastbee.data.controller.waterele;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.fastbee.common.annotation.Log;
import com.fastbee.common.core.controller.BaseController;
import com.fastbee.common.core.domain.AjaxResult;
import com.fastbee.common.enums.BusinessType;
import com.fastbee.waterele.domain.MaRechargerecord;
import com.fastbee.waterele.service.IMaRechargerecordService;
import com.fastbee.common.utils.poi.ExcelUtil;
import com.fastbee.common.core.page.TableDataInfo;

/**
 * 充值记录Controller
 *
 * @author kerwincui
 * @date 2024-08-12
 */
@RestController
@RequestMapping("/waterele/rechargerecord")
@Api(tags = "充值记录")
public class MaRechargerecordController extends BaseController
{
    @Autowired
    private IMaRechargerecordService maRechargerecordService;

/**
 * 查询充值记录列表
 */
@PreAuthorize("@ss.hasPermi('waterele:rechargerecord:list')")
@GetMapping("/list")
@ApiOperation("查询充值记录列表")
    public TableDataInfo list(MaRechargerecord maRechargerecord)
    {
        startPage();
        List<MaRechargerecord> list = maRechargerecordService.selectMaRechargerecordList(maRechargerecord);
        return getDataTable(list);
    }

    /**
     * 导出充值记录列表
     */
    @ApiOperation("导出充值记录列表")
    @PreAuthorize("@ss.hasPermi('waterele:rechargerecord:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, MaRechargerecord maRechargerecord)
    {
        List<MaRechargerecord> list = maRechargerecordService.selectMaRechargerecordList(maRechargerecord);
        ExcelUtil<MaRechargerecord> util = new ExcelUtil<MaRechargerecord>(MaRechargerecord.class);
        util.exportExcel(response, list, "充值记录数据");
    }

    /**
     * 获取充值记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('waterele:rechargerecord:query')")
    @GetMapping(value = "/{id}")
    @ApiOperation("获取充值记录详细信息")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(maRechargerecordService.selectMaRechargerecordById(id));
    }

    /**
     * 新增充值记录
     */
    @PreAuthorize("@ss.hasPermi('waterele:rechargerecord:add')")
    @PostMapping
    @ApiOperation("新增充值记录")
    public AjaxResult add(@RequestBody MaRechargerecord maRechargerecord)
    {
        return toAjax(maRechargerecordService.insertMaRechargerecord(maRechargerecord));
    }

    /**
     * 修改充值记录
     */
    @PreAuthorize("@ss.hasPermi('waterele:rechargerecord:edit')")
    @PutMapping
    @ApiOperation("修改充值记录")
    public AjaxResult edit(@RequestBody MaRechargerecord maRechargerecord)
    {
        return toAjax(maRechargerecordService.updateMaRechargerecord(maRechargerecord));
    }

    /**
     * 删除充值记录
     */
    @PreAuthorize("@ss.hasPermi('waterele:rechargerecord:remove')")
    @DeleteMapping("/{ids}")
    @ApiOperation("删除充值记录")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(maRechargerecordService.deleteMaRechargerecordByIds(ids));
    }
}
