package com.fastbee.iot.anfang.service.impl;

import com.fastbee.iot.anfang.service.IUploadedPhotosService;
import com.fastbee.iot.mapper.UploadedPhotosMapper;
import com.fastbee.iot.model.anfang.UploadedPhotos;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 存储上传的照片信息Service业务层处理
 * 
 * @author kerwincui
 * @date 2024-06-20
 */
@Service
public class UploadedPhotosServiceImpl implements IUploadedPhotosService
{
    @Resource
    private UploadedPhotosMapper uploadedPhotosMapper;

    /**
     * 查询存储上传的照片信息
     * 
     * @param id 存储上传的照片信息主键
     * @return 存储上传的照片信息
     */
    @Override
    public UploadedPhotos selectUploadedPhotosById(Long id)
    {
        return uploadedPhotosMapper.selectUploadedPhotosById(id);
    }

    /**
     * 查询存储上传的照片信息列表
     * 
     * @param uploadedPhotos 存储上传的照片信息
     * @return 存储上传的照片信息
     */
    @Override
    public List<UploadedPhotos> selectUploadedPhotosList(UploadedPhotos uploadedPhotos)
    {
        return uploadedPhotosMapper.selectUploadedPhotosList(uploadedPhotos);
    }

    /**
     * 新增存储上传的照片信息
     * 
     * @param uploadedPhotos 存储上传的照片信息
     * @return 结果
     */
    @Override
    public int insertUploadedPhotos(UploadedPhotos uploadedPhotos)
    {
        return uploadedPhotosMapper.insertUploadedPhotos(uploadedPhotos);
    }

    /**
     * 修改存储上传的照片信息
     * 
     * @param uploadedPhotos 存储上传的照片信息
     * @return 结果
     */
    @Override
    public int updateUploadedPhotos(UploadedPhotos uploadedPhotos)
    {
        return uploadedPhotosMapper.updateUploadedPhotos(uploadedPhotos);
    }

    /**
     * 批量删除存储上传的照片信息
     * 
     * @param ids 需要删除的存储上传的照片信息主键
     * @return 结果
     */
    @Override
    public int deleteUploadedPhotosByIds(Long[] ids)
    {
        return uploadedPhotosMapper.deleteUploadedPhotosByIds(ids);
    }

    /**
     * 删除存储上传的照片信息信息
     * 
     * @param id 存储上传的照片信息主键
     * @return 结果
     */
    @Override
    public int deleteUploadedPhotosById(Long id)
    {
        return uploadedPhotosMapper.deleteUploadedPhotosById(id);
    }
}
