package com.fastbee.waterele.service.impl;

import java.util.List;
import com.fastbee.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.waterele.mapper.MaGuangaiRecordMapper;
import com.fastbee.waterele.domain.MaGuangaiRecord;
import com.fastbee.waterele.service.IMaGuangaiRecordService;

/**
 * 灌溉记录Service业务层处理
 * 
 * @author kerwincui
 * @date 2024-08-12
 */
@Service
public class MaGuangaiRecordServiceImpl implements IMaGuangaiRecordService 
{
    @Autowired
    private MaGuangaiRecordMapper maGuangaiRecordMapper;

    /**
     * 查询灌溉记录
     * 
     * @param id 灌溉记录主键
     * @return 灌溉记录
     */
    @Override
    public MaGuangaiRecord selectMaGuangaiRecordById(Long id)
    {
        return maGuangaiRecordMapper.selectMaGuangaiRecordById(id);
    }

    /**
     * 查询灌溉记录列表
     * 
     * @param maGuangaiRecord 灌溉记录
     * @return 灌溉记录
     */
    @Override
    public List<MaGuangaiRecord> selectMaGuangaiRecordList(MaGuangaiRecord maGuangaiRecord)
    {
        return maGuangaiRecordMapper.selectMaGuangaiRecordList(maGuangaiRecord);
    }

    /**
     * 新增灌溉记录
     * 
     * @param maGuangaiRecord 灌溉记录
     * @return 结果
     */
    @Override
    public int insertMaGuangaiRecord(MaGuangaiRecord maGuangaiRecord)
    {
        maGuangaiRecord.setCreateTime(DateUtils.getNowDate());
        return maGuangaiRecordMapper.insertMaGuangaiRecord(maGuangaiRecord);
    }

    /**
     * 修改灌溉记录
     * 
     * @param maGuangaiRecord 灌溉记录
     * @return 结果
     */
    @Override
    public int updateMaGuangaiRecord(MaGuangaiRecord maGuangaiRecord)
    {
        maGuangaiRecord.setUpdateTime(DateUtils.getNowDate());
        return maGuangaiRecordMapper.updateMaGuangaiRecord(maGuangaiRecord);
    }

    /**
     * 批量删除灌溉记录
     * 
     * @param ids 需要删除的灌溉记录主键
     * @return 结果
     */
    @Override
    public int deleteMaGuangaiRecordByIds(Long[] ids)
    {
        return maGuangaiRecordMapper.deleteMaGuangaiRecordByIds(ids);
    }

    /**
     * 删除灌溉记录信息
     * 
     * @param id 灌溉记录主键
     * @return 结果
     */
    @Override
    public int deleteMaGuangaiRecordById(Long id)
    {
        return maGuangaiRecordMapper.deleteMaGuangaiRecordById(id);
    }
}
