package com.fastbee.waterele.service.impl;

import java.util.List;
import com.fastbee.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.waterele.mapper.MaWatereleRecordMapper;
import com.fastbee.waterele.domain.MaWatereleRecord;
import com.fastbee.waterele.service.IMaWatereleRecordService;

/**
 * 水电双计数据记录Service业务层处理
 * 
 * @author kerwincui
 * @date 2024-08-12
 */
@Service
public class MaWatereleRecordServiceImpl implements IMaWatereleRecordService 
{
    @Autowired
    private MaWatereleRecordMapper maWatereleRecordMapper;

    /**
     * 查询水电双计数据记录
     * 
     * @param id 水电双计数据记录主键
     * @return 水电双计数据记录
     */
    @Override
    public MaWatereleRecord selectMaWatereleRecordById(Long id)
    {
        return maWatereleRecordMapper.selectMaWatereleRecordById(id);
    }

    /**
     * 查询水电双计数据记录列表
     * 
     * @param maWatereleRecord 水电双计数据记录
     * @return 水电双计数据记录
     */
    @Override
    public List<MaWatereleRecord> selectMaWatereleRecordList(MaWatereleRecord maWatereleRecord)
    {
        return maWatereleRecordMapper.selectMaWatereleRecordList(maWatereleRecord);
    }

    /**
     * 新增水电双计数据记录
     * 
     * @param maWatereleRecord 水电双计数据记录
     * @return 结果
     */
    @Override
    public int insertMaWatereleRecord(MaWatereleRecord maWatereleRecord)
    {
        maWatereleRecord.setCreateTime(DateUtils.getNowDate());
        return maWatereleRecordMapper.insertMaWatereleRecord(maWatereleRecord);
    }

    /**
     * 修改水电双计数据记录
     * 
     * @param maWatereleRecord 水电双计数据记录
     * @return 结果
     */
    @Override
    public int updateMaWatereleRecord(MaWatereleRecord maWatereleRecord)
    {
        maWatereleRecord.setUpdateTime(DateUtils.getNowDate());
        return maWatereleRecordMapper.updateMaWatereleRecord(maWatereleRecord);
    }

    /**
     * 批量删除水电双计数据记录
     * 
     * @param ids 需要删除的水电双计数据记录主键
     * @return 结果
     */
    @Override
    public int deleteMaWatereleRecordByIds(Long[] ids)
    {
        return maWatereleRecordMapper.deleteMaWatereleRecordByIds(ids);
    }

    /**
     * 删除水电双计数据记录信息
     * 
     * @param id 水电双计数据记录主键
     * @return 结果
     */
    @Override
    public int deleteMaWatereleRecordById(Long id)
    {
        return maWatereleRecordMapper.deleteMaWatereleRecordById(id);
    }
}
